/* 
    Copyright (C) 2004  Mika Raento - Renaud Petit

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


    email: mraento@cs.helsinki.fi - petit@cs.helsinki.fi 
*/


//CContextNotifyClientSession.cpp

#include "NotifyCommon.h"
#include "ContextNotifyClientSession.h"
#include <e32math.h>
#include <eikenv.h>
#include "list.h"
#include "app_context.h"
#include <eikspane.h>
#include <eikimage.h>
#include <avkon.hrh>
#include <avkon.rsg>
#include <bautils.h>
#include "break.h"
#include "server_startup.h"

#ifdef __S60V2__
#include <AknsControlContext.h>
#include <AknsBasicBackgroundControlContext.h>
#include <AknsDrawUtils.h>
#endif

static const TUint KDefaultMessageSlots = 2;
static const TUid KServerUid3 = {0x10204BAA}; // matches UID in server/group/ContextNotifyer.mbm file

_LIT(KContextNotifyFilename, "ContextNotify");

/*
 * Concepts:
 * !Customizing the status pane!
 */

class MObjectProviderEx
        {
public:
        template<class T>
        T* MopGetObject(T*& aPtr)
                { return (aPtr=static_cast<T*>(MopGetById(T::ETypeId))); }

private: // must be overridden
        virtual TTypeUid::Ptr MopSupplyObject(TTypeUid aId) = 0;

private: // may be overridden to continue chain of responsibility
        IMPORT_C virtual MObjectProvider* MopNext();

private: // internal implementation
        IMPORT_C TAny* MopGetById(TTypeUid aId);
	friend class CLocalNotifyWindow;
	friend class CLocalNotifyWindowImpl;
        };

class CLocalNotifyWindowImpl : public CLocalNotifyWindow, 
		public MContextBase {
	CLocalNotifyWindowImpl() { }

	~CLocalNotifyWindowImpl() {
		CALLSTACKITEM_N(_CL("CLocalNotifyWindowImpl"), _CL("CLocalNotifyWindowImpl"));

#ifdef __S60V2__
		delete iBackground;
#endif
		if (iResource) iEikonEnv->DeleteResourceFile(iResource);

		CList<TSlot>::Node	*i;
		for (i=iSlots->iFirst; i; i=i->Next) {
			delete i->Item.iImage;
		}
		delete iSlots;
		delete iToBeNotified;
	}

	void SetPositions() {
		iPrevIndex=0; iPrevNode=iSlots->iFirst;

		TInt xpos=Rect().Width()-9, ypos=0;

		CList<TSlot>::Node	*i;
		for (i=iSlots->iFirst; i; i=i->Next) {
			ypos += 1;
			i->Item.iImage->SetPosition(TPoint(xpos, ypos));
			ypos += i->Item.iImage->Size().iHeight;
		}
	}

	void DrawOnGc(CWindowGc& gc, TPoint& aFromPos) {
		TInt xpos=aFromPos.iX, ypos=aFromPos.iY;

		CList<TSlot>::Node	*i;
		for (i=iSlots->iFirst; i; i=i->Next) {
			ypos += 1;
			TRect r( TPoint(0, 0), i->Item.iImage->Bitmap()->SizeInPixels());
			gc.BitBltMasked(TPoint(xpos, ypos), 
				i->Item.iImage->Bitmap(), r, i->Item.iImage->Mask(), ETrue);

			ypos += i->Item.iImage->Size().iHeight;
		}
	}
	
#ifdef __S60V2__
	virtual void Draw(const TRect& aRect) const {
		CALLSTACKITEM_N(_CL("CLocalNotifyWindowImpl"), _CL("Draw"));
		CWindowGc& gc = SystemGc();

		if (iBackground) {
			//AknsDrawUtils::Background( AknsUtils::SkinInstance(), iBackground, 
				//this, gc, aRect );
			AknsDrawUtils::Background( AknsUtils::SkinInstance(), iBackground, gc, aRect );
		} else {
			gc.Clear();
		}
	}
	TRect	iPaneRect;
	MObjectProviderEx* iParent;
	CAknsBasicBackgroundControlContext* iBackground;

	virtual void SizeChanged() {
		CAknTitlePane::SizeChanged();
		TRect r=Rect();
		r.Move( -iPaneRect.iTl.iX, 0 );
		r.Resize( iPaneRect.iTl.iX, 0 );
		iBackground->SetRect( r );
	}
#endif
	void ConstructL() {
		CALLSTACKITEM_N(_CL("CLocalNotifyWindowImpl"), _CL("ConstructL"));

#ifdef __S60V2__
		iBackground=CAknsBasicBackgroundControlContext::NewL( KAknsIIDQsnBgAreaStatus,
			TRect(0, 0, 0, 0), EFalse );
		//iBackground->SetParentPos( iPaneRect.iTl );
#endif
		TFileName resfile=_L("z:\\System\\data\\avkon.rsc");
		BaflUtils::NearestLanguageFile(iEikonEnv->FsSession(), resfile); //for localization
		CC_TRAPD(err, iResource=iEikonEnv->AddResourceFileL(resfile));
		User::LeaveIfError(err);
		{
			HBufC8* res = 
				iEikonEnv->AllocReadResourceAsDes8LC(R_AVKON_STATUS_PANE_TITLE_DEFAULT);
			TResourceReader theReader;
			theReader.SetBuffer(res);
			CAknTitlePane::ConstructFromResourceL(theReader);
			CleanupStack::PopAndDestroy();
		}
		iSp=iEikonEnv->AppUiFactory()->StatusPane();
		iSlots=CList<TSlot>::NewL();
		iToBeNotified=CList<CCoeControl*>::NewL();
	}

	void Redraw() {
		CALLSTACKITEM_N(_CL("CLocalNotifyWindowImpl"), _CL("Redraw"));
		iSp=0;
		if (iEikonEnv->AppUiFactory())
			iSp=iEikonEnv->AppUiFactory()->StatusPane();
		if (iSp) iSp->DrawNow();
		CList<CCoeControl*>::Node *i;
		for (i=iToBeNotified->iFirst; i; i=i->Next) {
			i->Item->DrawNow();
		}
	}
	void SetTitleText(HBufC *aText) {
		SetText(aText);
		Redraw();
	}
	void SetTitleTextToDefaultL() {
		SetTextToDefaultL();
		Redraw();
	}
	virtual TInt AddIconL(CFbsBitmap* aIcon, CFbsBitmap* aMask) {
		CALLSTACKITEM_N(_CL("CLocalNotifyWindowImpl"), _CL("AddIconL"));

		CFbsBitmap* bm=new (ELeave) CFbsBitmap;
		CleanupStack::PushL(bm);
		User::LeaveIfError(bm->Duplicate(aIcon->Handle()));
		CFbsBitmap* mask=new (ELeave) CFbsBitmap;
		CleanupStack::PushL(mask);
		User::LeaveIfError(mask->Duplicate(aMask->Handle()));
		TInt id=iLastId++;

		CEikImage* im=new (ELeave) CEikImage();
		im->SetPicture(bm, mask);
		im->SetPictureOwnedExternally(EFalse);
		CleanupStack::Pop(2);
		CleanupStack::PushL(im);
		im->SetSize(bm->SizeInPixels());

		iSlots->AppendL(TSlot(id, im));
		CleanupStack::Pop(1);

		SetPositions();
		Redraw();
		return id;

	}
	virtual void ChangeIconL(CFbsBitmap* aIcon, CFbsBitmap* aMask, TInt aId)
	{
		CALLSTACKITEM_N(_CL("CLocalNotifyWindowImpl"), _CL("ChangeIconL"));

		CList<TSlot>::Node	*i;
		for (i=iSlots->iFirst; i; i=i->Next) {
			if (i->Item.iId==aId) break;
		}
		if (!i) User::Leave(KErrNotFound);
		CFbsBitmap* bm=new (ELeave) CFbsBitmap;
		CleanupStack::PushL(bm);
		User::LeaveIfError(bm->Duplicate(aIcon->Handle()));
		CFbsBitmap* mask=new (ELeave) CFbsBitmap;
		CleanupStack::PushL(mask);
		User::LeaveIfError(mask->Duplicate(aMask->Handle()));

		CEikImage* im=new (ELeave) CEikImage();
		im->SetPicture(bm, mask);
		im->SetPictureOwnedExternally(EFalse);
		im->SetSize(bm->SizeInPixels());
		CleanupStack::Pop(2);
		CleanupStack::PushL(im);

		delete i->Item.iImage;
		i->Item.iImage=im;
		CleanupStack::Pop(1);

		SetPositions();
		Redraw();
	}
	virtual void RemoveIcon(TInt aId) {
		CALLSTACKITEM_N(_CL("CLocalNotifyWindowImpl"), _CL("RemoveIcon"));

		CList<TSlot>::Node	*i;
		for (i=iSlots->iFirst; i; i=i->Next) {
			if (i->Item.iId==aId) {
				delete i->Item.iImage;
				iSlots->DeleteNode(i, ETrue);
				break;
			}
		}
		iPrevNode=0; iPrevIndex=0;

		SetPositions();
		Redraw();
	}


	TInt CountComponentControls() const {
		CALLSTACKITEM_N(_CL("CLocalNotifyWindowImpl"), _CL("CountComponentControls"));
		TInt ret=CAknTitlePane::CountComponentControls();
		return ret+iSlots->iCount;
	}
	CCoeControl* ComponentControl(TInt aIndex) const {
		CALLSTACKITEM_N(_CL("CLocalNotifyWindowImpl"), _CL("ComponentControl"));
		if (aIndex < CAknTitlePane::CountComponentControls()) {
			return CAknTitlePane::ComponentControl(aIndex);
		} else {
			aIndex-=CAknTitlePane::CountComponentControls();
			if (aIndex==0) {
				iPrevNode=iSlots->iFirst;
				iPrevIndex=0;
			} else {
				while (iPrevIndex < aIndex) {
					iPrevNode=iPrevNode->Next;
					iPrevIndex++;
				}
			}
			return iPrevNode->Item.iImage;
		}
	}

	virtual void AddNotifiedControl(CCoeControl* aControl) {
		iToBeNotified->AppendL(aControl);
	}
	virtual void RemoveNotifiedControl(CCoeControl* aControl) {
		CList<CCoeControl*>::Node *i;
		for (i=iToBeNotified->iFirst; i; i=i->Next) {
			if (i->Item == aControl) {
				iToBeNotified->DeleteNode(i, ETrue);
				return;
			}
		}
	}

	TInt	iLastId;
	mutable TInt	iRefCount;

	struct TSlot {
		TInt		iId;
		CEikImage*	iImage;

		TSlot() : iId(-1), iImage(0) { }
		TSlot(TInt aId, CEikImage* aImage) :
		iId(aId), iImage(aImage) { }
	};
	CList<TSlot>*	iSlots;
	CList<CCoeControl*> *iToBeNotified;
	CEikStatusPane* iSp;
	CAknTitlePane*	iPrevious;

	mutable CList<TSlot>::Node *iPrevNode; 
	mutable TInt iPrevIndex;
	TInt iResource;

	friend class CLocalNotifyWindow;
};

EXPORT_C void CNotifyWindowControl::Draw(const TRect& ) const
{
	CWindowGc& gc = SystemGc();
	TPoint pos=Position();
	iLocal->DrawOnGc(gc, pos);
}

EXPORT_C void CNotifyWindowControl::ConstructL(CCoeControl *aTopLevel)
{
	iTopLevel=aTopLevel;
	iLocal=CLocalNotifyWindow::Global();
	if (iLocal) iLocal->AddNotifiedControl(iTopLevel);
}

EXPORT_C CNotifyWindowControl::~CNotifyWindowControl()
{
	if (iLocal)  {
		iLocal->RemoveNotifiedControl(iTopLevel);
	}
}

CLocalNotifyWindow::~CLocalNotifyWindow() { }

EXPORT_C CLocalNotifyWindow* CLocalNotifyWindow::Global()
{
	if (! CEikonEnv::Static()) return 0;
	CLocalNotifyWindowImpl* w=(CLocalNotifyWindowImpl*)Dll::Tls();
	return w;
}

EXPORT_C void CLocalNotifyWindow::Destroy()
{
	if (! CEikonEnv::Static()) return;
	CLocalNotifyWindowImpl* w=(CLocalNotifyWindowImpl*)Dll::Tls();

	if (!w) return;
	CEikStatusPane* sp=CEikonEnv::Static()->AppUiFactory()->StatusPane();
	sp->SwapControlL( TUid::Uid(EEikStatusPaneUidTitle), w->iPrevious);
	User::LeaveIfError(Dll::SetTls(0));
	delete w;
}


EXPORT_C void CLocalNotifyWindow::CreateAndActivateL()
{
	if (! CEikonEnv::Static()) return;
	void* existing=Dll::Tls();
	CLocalNotifyWindowImpl* w=0;
	if (existing) {
		User::Leave(KErrAlreadyExists);
	} else {
		auto_ptr<CLocalNotifyWindowImpl> wp(new (ELeave) CLocalNotifyWindowImpl);
		w=wp.get();

		MEikAppUiFactory* factory=CEikonEnv::Static()->AppUiFactory();
		CEikStatusPane* sp=factory->StatusPane();
		CCoeAppUiBase* appui=CEikonEnv::Static()->AppUi();

		CAknTitlePane* tp=w;
		CAknTitlePane* oldtitle=(CAknTitlePane*)
			sp->ControlL( TUid::Uid(EEikStatusPaneUidTitle));

		MObjectProvider* p1=oldtitle;
		MObjectProviderEx* p2=(MObjectProviderEx*)p1;
		MObjectProvider* p=p2->MopNext();

#ifndef __WINS__
// crashes on 2.0 emu on redraw?
		w->SetMopParent(p);
#endif
#ifdef __S60V2__
		w->iPaneRect=sp->PaneRectL(TUid::Uid(EEikStatusPaneUidTitle));
		w->iParent=(MObjectProviderEx*)p;
#endif
		w->ConstructL();

		w->SetTextL(*oldtitle->Text());

		// FIXME: the skin background is drawn as if the title pane
		// is at the top left corner, instead of in the middle of the screen.
		// I don't know how to fix it yet :-( -MR

		TRect r=oldtitle->Rect();
		w->SetRect( r );
		User::LeaveIfError( Dll::SetTls(w) );
		wp.release();
		sp->SwapControlL( TUid::Uid(EEikStatusPaneUidTitle), tp);
		w->iPrevious=oldtitle;
	}
}

EXPORT_C RContextNotifyClientSession::RContextNotifyClientSession() : RSessionBase(), iIdPckg(0)
{
	
}

EXPORT_C TInt RContextNotifyClientSession::Connect()
{
	TInt result;
	
	CC_TRAP(result, StartServerL(KContextNotifyName, KServerUid3, KContextNotifyFilename));
	if (result == KErrNone)
	{
		result = CreateSession(KContextNotifyName,
			Version(),
			KDefaultMessageSlots);	
	}
	return result;
}

EXPORT_C void RContextNotifyClientSession::Close()
{
	RSessionBase::Close();
	delete iIdPckg; iIdPckg=0;
}

EXPORT_C TVersion RContextNotifyClientSession::Version() const
{
	return(TVersion(KContextNotifyMajorVersionNumber,
		KContextNotifyMinorVersionNumber,
		KContextNotifyBuildVersionNumber));
}

//----------------------------------------------------------------------------------------

EXPORT_C void RContextNotifyClientSession::Cancel() const
{
	SendReceive(ECancel, NULL);
}

EXPORT_C void RContextNotifyClientSession::TerminateContextNotify(TRequestStatus& aStatus)
{
	SendReceive(ETerminateContextNotify, NULL, aStatus);
}

EXPORT_C void RContextNotifyClientSession::AddIcon(CFbsBitmap* aIcon, CFbsBitmap* aMask, TInt& aId, TRequestStatus& aStatus)
{
	delete iIdPckg; iIdPckg=0; iIdPckg=new (ELeave) TPckg<TInt>(aId);

	TAny* messageParameters[KMaxMessageArguments];
	messageParameters[0] = (void*)aIcon->Handle();
	messageParameters[1] = (void*)aMask->Handle();
	messageParameters[2] = (void*)iIdPckg;

	SendReceive(EAddIcon, &messageParameters[0], aStatus);
}

EXPORT_C void RContextNotifyClientSession::RemoveIcon(TInt aId, TRequestStatus& aStatus)
{
	TAny* messageParameters[KMaxMessageArguments];
	messageParameters[0] = (void*)aId;
	SendReceive(ERemoveIcon, &messageParameters[0], aStatus);
}

EXPORT_C void RContextNotifyClientSession::ChangeIcon(CFbsBitmap* aIcon, CFbsBitmap* aMask, TInt aId, TRequestStatus& aStatus)
{
	TAny* messageParameters[KMaxMessageArguments];
	messageParameters[0] = (void*)aIcon->Handle();
	messageParameters[1] = (void*)aMask->Handle();
	messageParameters[2] = (void*)aId;

	SendReceive(EChangeIcon, &messageParameters[0], aStatus);
}
