/* 
    Copyright (C) 2004  Mika Raento - Renaud Petit

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


    email: mraento@cs.helsinki.fi - petit@cs.helsinki.fi 
*/

#ifndef CONTEXT_LOGAPPUI_H
#define CONTEXT_LOGAPPUI_H

#define NULL 0

#include "ver.h"

// INCLUDES
#include <eikapp.h>
#include <eikdoc.h>
#include <e32std.h>
#include <coeccntx.h>

#include "presence_publisher.h"

#include "timeout.h"

#include <sendas.h>

#include "csd_cell.h"
#include "bb_logger.h"

class CSmsSocket;
class CSmsSnapshot;

class CApSettingMaintainer;
class CNotifyState;
class CBTDeviceList;
class CPresenceDetailView;
class CPresenceDescriptionView;

#include "appuibase.h"
// CONSTANTS
//const ?type ?constant_var = ?constant;


// CLASS DECLARATION

/**
* Application UI class.
* Provides support for the following features:
* - EIKON control architecture
* 
*/
class CContext_logAppUi : public CContextLogAppUiBase,
	public MSendAsObserver, 
	public MPresencePublisherListener, 
	public MSettingListener
{
public: // // Constructors and destructor
	
	CContext_logAppUi(MApp_context& Context) : CContextLogAppUiBase(Context) { }
        /**
        * EPOC default constructor.
        */      
        void ConstructL();
	
        /**
        * Destructor.
        */      
        ~CContext_logAppUi();
        
public: // New functions
	void SetUserGiven();
	void RestoreUserGiven();
	
public: // Functions from base classes
	
private:
        // From MEikMenuObserver
        void DynInitMenuPaneL(TInt aResourceId,CEikMenuPane* aMenuPane);
public:
	// From MPresencPublisherListener
	void NotifyNewPresence(const CBBPresence*);

private:
	
	// MSettingListener
	virtual void SettingChanged(TInt Setting);

        /**
        * From CEikAppUi, takes care of command handling.
        * @param aCommand command to be handled
        */
        void HandleCommandL(TInt aCommand);
private:

public:
	void DialogTest();	
	void PresenceDetails();
	void PresenceDescription();
private:
	// MSendAsObserver
	virtual TBool CapabilityOK(TUid /*aCapabilty*/, TInt /*aResponse*/) { return ETrue; }

private: //Data
        
	HBufC * iCaptionMyContext;

	class CPresencePublisher* iPresencePublisher;
	class CCircularLog * iUserContextLog;
	HBufC* iUserContext;

	//log_test* lt;
	RThread* t;
	
	class Clog_comm* comml;
	
	class CSendUITransfer* transferer;

	class CAknGlobalNote* globalNote;
	
	
	bool case_closed;
	bool ringing;

	class CBBLogger*	bbl;

	class CCall_record *in_recorder, *out_recorder;

	class CPeriodicTransfer* iPeriodicTransfer;
	class CConnectionOpener* iConnectionOpener;

	RSocketServ	iServ;
#ifdef __S60V2__
	RConnection	iConnection;
#endif

	TBuf<6> prev; TBuf<30> not_avail;

	class CPresenceDetailView	*iPresenceDetailView; // OWNED BY FRAMEWORK
	class CPresenceDescriptionView	*iPresenceDescriptionView; // OWNED BY FRAMEWORK
	class CBTDevApView		*iBTDevApView, *iBTBuddiesView; // OWNED BY FRAMEWORK
	class CUserView* iUserView;

	RFile			iTestFile; bool iTestFileOpen;

	class CNotifyState		*iContextRunning, *iLoggingRunning;
	class CLocaSender	*iLocaSender;
	class CSmsStatusReplier	*iSmsStatus;

	class CBBDumper		*iDumper;
	class CBBLocalRunner	*iBBLocal;
	class CSocketTestReader	*iSocketTestReader;

	class CMediaTest	*iMediaTest;
};

#endif

// End of File
