/* 
    Copyright (C) 2004  Mika Raento - Renaud Petit

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


    email: mraento@cs.helsinki.fi - petit@cs.helsinki.fi 
*/

// INCLUDE FILES
#include "ContextFlickrDocument.h"
#include "ContextFlickrAppUi.h"
#include "app_context_impl.h"
#include "symbian_auto_ptr.h"
#include "splash.h"
#include "reporting.h"
#include "break.h"
#include "callstack.h"

// ================= MEMBER FUNCTIONS =======================

// constructor
CContextFlickrDocument::CContextFlickrDocument(CEikApplication& aApp)
: CAknDocument(aApp)    
{
}

// destructor
CContextFlickrDocument::~CContextFlickrDocument()
{
}

// EPOC default constructor can leave.
void CContextFlickrDocument::ConstructL()
{
	TBuf<50> msg;
	TRAPD(err, iSplash=CSplashScreen::NewL());
	if (err==KErrNone) {
		TRAP(err, 
			iSplash->ConstructL(CContextFlickrAppUi::InitializationSteps()+
			MContextDocument::InitializationSteps()));
		if (err==KErrNone) {
			iSplash->MakeVisible(ETrue);
			iSplash->ActivateL();
			iSplash->DrawNow();
		} else {
			delete iSplash; iSplash=0;
			msg=_L("Failed to construct splash ");
			msg.AppendNum(err);
		}
	} else {
		msg=_L("Failed to create splash ");
		msg.AppendNum(err);
	}

	MContextDocument::ConstructL(Application(),
		iDefaultSettings,
		KCLSettingsTuple,
		_L("flickr_app.txt"),
		_L("context_log"));

#ifdef __WINS__
	iContext->SetAppDir(_L("c:\\system\\apps\\contextflickr\\"));
#endif
	if (msg.Length()>0) iContext->Reporting().DebugLog(msg);
}

// Two-phased constructor.
CContextFlickrDocument* CContextFlickrDocument::NewL(
						 CEikApplication& aApp)     // CContextFlickrApp reference
{
	auto_ptr<CContextFlickrDocument> self(new (ELeave) CContextFlickrDocument( aApp ));
	self->ConstructL();
	
	return self.release();
}

// ----------------------------------------------------
// CContextFlickrDocument::CreateAppUiL()
// constructs CContextFlickrAppUi
// ----------------------------------------------------
//
CEikAppUi* CContextFlickrDocument::CreateAppUiL()
{
	CContextFlickrAppUi* ret=new (ELeave) CContextFlickrAppUi(*iContext, iSplash);
	return ret;
}

void CContextFlickrDocument::StepDone()
{
	if (iSplash) iSplash->StepDone();
}

// End of File  
